'use strict';

var fs			= require('fs');
var path		= require('path');
var _			= require('underscore');
var inquirer	= require('inquirer');
var request		= require('request');
var open		= require('open');
var archiver 	= require('archiver');
var semver		= require('semver');
var shelljs 	= require('shelljs');
var open        = require('open');

var create = function(appPath) {
	// init project at given path or default to cwd
	appPath = (typeof appPath == 'string') ? appPath : process.cwd();

	// check if chosen path exists and is a directory
	if( !fs.existsSync(appPath) ) return console.error("Error: path does not exist");
	if( !fs.lstatSync(appPath).isDirectory() ) return console.error("Error: path is not a directory");

	// ask some questions to setup the project
	inquirer.prompt([
		{
			type: "input",
			name: "id",
			message: "What is your app ID? (can be found in the developers center)",
		},
		{
			type: "name",
			name: "name",
			message: "What is the name of your app?",
			default: "myFormideApp"
		},
		{
			type: "description",
			name: "description",
			message: "What is the description of your app?",
			default: "This is an awesome app!"
		},
		{
			type: "input",
			name: "version",
			message: "What is you app version? (fill in a semver compatible version number)",
			default: "1.0.0"
		},
		{
			type: "confirm",
			name: "confirm",
			message: "Use this config?"
		}
	], function(input) {
		var projectPath = path.join(appPath, input.name);

		// check if version valid
		if(semver.valid(input.version) == null) {
			return console.log("Version number invalid, make sure you use a valid semver version number".red);
		}

		// check if dir already exists
		if (fs.existsSync(projectPath)) return console.log(("Error: path " + projectPath + " already exist").red);

		// create project dir
		fs.mkdirSync(projectPath);

		// build manifest
		var manifest = {
			id: input.id,
			name: input.name,
			version: input.version
		};

		// add author when logged in
		if (typeof lib.settings.user == 'object') {
			manifest.author = {
				name: lib.settings.user.username
			};
		}

		// create app.json
		fs.writeFileSync(path.join(projectPath, 'app.json'), JSON.stringify(manifest, null, 4));

		// create README.md
		fs.writeFileSync(path.join(projectPath, 'README.md'), "# " + input.name + "\n" +input.description);
	});
};

var info = function(appPath) {
	// run project at given path or default to cwd
	appPath = (typeof appPath == 'string') ? appPath : process.cwd();

	// get manifest contents for upload info
	try {
		var manifest = fs.readFileSync(appPath + '/app.json', 'utf8');
		manifest = JSON.parse(manifest);

		// show manifest contents
		manifest.author = manifest.author.name;
		console.log(columnify(manifest, {
			showHeaders: false
		}));
	}
	catch(e) {
		if (e.code === 'ENOENT') return console.error('No app info found in current folder'.red);
		return console.error(('An unkonwn error occured :' + e.message).red);
	}
};

var run = function(appPath) {
	// run project at given path or default to cwd
	appPath = (typeof appPath == 'string') ? appPath : process.cwd();

	// open local app index.html in browser
	open('file://' + appPath + '/index.html');

	// wait a bit and exit
	setTimeout(process.exit, 250);
};

var bump = function(level) {
	var appPath = process.cwd();
	level = (typeof level == 'string') ? level : 'patch';

	// get manifest contents for bump
	var manifest = fs.readFileSync(appPath + '/app.json', 'utf8');

	try {
		manifest = JSON.parse(manifest);
	}
	catch (e) {
		return console.log(('Error parsing app manifest: ' + e.message).red);
	}

	// patch bump manifest version number
	manifest.version = semver.inc(manifest.version, level);
	fs.writeFileSync(appPath + '/app.json', JSON.stringify(manifest), 'utf8');

	console.log(('App version bumped to ' + manifest.version).green);
}

var publish = function(appPath) {
	// publish project at given path or default to cwd
	appPath = (typeof appPath == 'string') ? appPath : process.cwd();

	// get manifest contents for upload info
	var manifest = fs.readFileSync(appPath + '/app.json', 'utf8');

	try {
		manifest = JSON.parse(manifest);
	}
	catch (e) {
		return console.log(('Error parsing app manifest: ' + e.message).red);
	}

	// compress and upload app
	compress(appPath, manifest, function(err, zipPath) {
		upload(zipPath, manifest, function(err, httpResponse, body) {

			// remove app archive
			fs.unlink(zipPath);

			// handle upload error
			if (err) {
				return console.log(('Upload failed: ' + err.message).red);
			}

			try {
				var response = JSON.parse(body);

				// handle not 200 OK response from api as error
				if (httpResponse.statusCode !== 200) {
					return console.log(('Upload failed: ' + response.message).red);
				}

				// log success
				console.log(('Upload complete: ' + response.message).green);

				// wait a bit and exit
				setTimeout(process.exit, 1000);
			}
			catch (e) {
				return console.log(('Upload failed: ' + e.message).red);
			}
		});
	});
};

var compress = function(appPath, manifest, callback) {
	var zipPath = appPath + '/app.zip';
	var zip = fs.createWriteStream(zipPath);
	var archive = archiver('zip');

	archive
		.on('entry', function(entry) {
			console.log(('File processed: ' + entry.name).grey);
		})
		.on('error', function(err) {
			console.log(('Error archiving app: ' + err.message).red);
		})
		.bulk([{
			expand: true,
			cwd: appPath,
			src: ['**']
		}])
		.pipe(zip);

	// finalize archiving, this will trigger zip.on('close')
	archive.finalize();

	zip
		.on('error', function() {
			console.log(('Error writing app archive: ' + err.message).red);
		})
		.on('close', function() {
			console.log(('Finished processing files').green);
			return callback(null, zipPath);
		});
};

var upload = function(zipPath, manifest, callback) {
	lib.api.call({
		path: '/clients/' + manifest.id + '/versions',
		method: 'POST',
		json: false,
		appUpload: {
			version: manifest.version,
			zipPath: zipPath
		}
	}, callback);
};

module.exports.create = create;
module.exports.publish = publish;
module.exports.run = run;
module.exports.info = info;
module.exports.bump = bump;